from django.http import HttpResponse
from django.conf import settings
from django.contrib.auth.models import AnonymousUser
from apikey.models import ApiKey, Token
from datetime import datetime, timedelta


TOKEN_AUTH_HEADER = getattr(settings, 'TOKEN_AUTH_HEADER', 'X-Auth-Token')
TOKEN_VALID_SECONDS = getattr(settings, 'TOKEN_VALID_SECONDS', 3600)


class TokenAuthentication(object):

    def is_authenticated(self, request):
        " Check if request is authenticated."

        auth_header = ('Http-%s' % (
            TOKEN_AUTH_HEADER, )).upper().replace('-', '_')

        if auth_header not in request.META:
            return False

        auth_string = request.META.get(auth_header)

        limit = datetime.now() - timedelta(0, TOKEN_VALID_SECONDS)

        try:
            key = Token.objects.get(token=auth_string, last_used__gt=limit)

            # Update last_used.
            key.save()
            request.user = key.user
            return True

        except Token.DoesNotExist:
            request.user = AnonymousUser()
            return False

    def challenge(self):
        resp = HttpResponse('Authorization Required')
        resp.status_code = 401
        return resp


class ApiKeyAuthentication(object):

    def is_authenticated(self, request):
        " Check if request is authenticated."

        auth_header = getattr(
            settings, 'APIKEY_AUTHORIZATION_HEADER', 'Authorization')

        auth_header = ('Http-%s' % (auth_header, )).upper().replace('-', '_')

        if auth_header not in request.META:
            return False

        auth_string = request.META.get(auth_header)

        try:
            key = ApiKey.objects.get(key=auth_string, is_active=True)
            request.user = key.user
            return True
        except ApiKey.DoesNotExist:
            request.user = AnonymousUser()
            return False

    def challenge(self):
        resp = HttpResponse('Authorization Required')
        resp.status_code = 401
        return resp



from setuptools import setup, find_packages
from os import path
BASE_DIR = path.abspath(path.dirname(__file__))


setup(
    name='django-apikey',
    description='Simple Key and Token authentication for Django.',
    version='0.2.0',
    long_description=open(path.join(BASE_DIR, 'README.rst')).read(),
    url='https://www.github.com/jorgeecardona/django-apikey',
    packages=find_packages(),
    author='Jorge Eduardo Cardona',
    author_email='jorgeecardona@gmail.com',
    license='BSD',
    keywords='django authentication piston',
    classifiers=[
        'License :: OSI Approved :: BSD License',
        ],
    )



from django.db import models
from hashlib import sha1
from random import randint, choice
from string import letters, digits
from django.conf import settings

# Validate TOKEN_SIZE
TOKEN_SIZE = getattr(settings, 'TOKEN_SIZE', 60)


class ApiKeyManager(models.Manager):

    def _generate_key(self):
        return sha1(str(randint(0, (16 ** 40) - 1))).hexdigest()

    def generate(self, user):

        # Create
        key = self._generate_key()
        if self.filter(key=key).count() > 0:
            key = self._generate_key()

        return self.create(user=user, key=key)


class ApiKey(models.Model):
    " Api key model."

    #: User that owns the key.
    user = models.ForeignKey('auth.User', related_name='api_keys')

    #: Key.
    key = models.CharField(max_length=40)

    #: Is active?
    is_active = models.BooleanField(default=True)

    # Objects.
    objects = ApiKeyManager()


class TokenManager(models.Manager):

    def _generate(self):
        return ''.join([choice(letters + digits) for i in range(TOKEN_SIZE)])

    def create(self, user):
        token = self._generate()
        return super(TokenManager, self).create(user=user, token=token)


class Token(models.Model):
    " Token. "

    #: User that creates the token.
    user = models.ForeignKey('auth.User', related_name='api_tokens')

    #: Token.
    token = models.CharField(max_length=TOKEN_SIZE)

    #: IP Address that creates the token.
    ip = models.IPAddressField(null=True, default=None)

    #: Last used time.
    last_used = models.DateTimeField(auto_now=True)

    #: Objects Manager.
    objects = TokenManager()
